#!/usr/bin/python

# Copyright (C) 2011  OpenStack LLC
# Copyright (C) 2012  Nebula Inc.
# Author: Soren Hansen <soren@ubuntu.com>
#
# In part based on copy-proposed-kernel.py, which is written by
# Martin Pitt <martin.pitt@canonical.com> and Copyright (C) 2011,
# 2012  Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Copy a package from one archive to another."""


import apt_pkg
from launchpadlib.launchpad import Launchpad
from optparse import OptionParser
import ppalib
import sys

CONSUMER_KEY = "copy-ppa-pkg.py"


def parse_options(args=None):
    usage = ("%prog [options] <source archive> <destination archive> "
             "<series> <pkg>")
    description = ('Archive names can either be "<user>/<ppaname>" for PPAs, '
                   '"debian/release" for Debian proper, or "ubuntu/XXX" for '
                   'Ubuntu proper. In the latter case, XXX can be one of '
                   '"Release", "Updates", "Proposed", or "Security".')

    parser = OptionParser(usage=usage, description=description)

    parser.add_option("-l", "--launchpad", dest="launchpad_instance",
                      default="production")
    parser.add_option("-b", "--include-binaries", action="store_true",
                      dest="binaries", default=True,
                      help="Include binaries (this is the default)")
    parser.add_option("-r", "--rebuild", "--no-include-binaries",
                      action="store_false", dest="binaries",
                      help="Do not include binaries")
    parser.add_option("-s", "--to-series", metavar="SERIES",
                      help="Copy to SERIES. Default is same series as source.")
    parser.add_option("-f", "--force", action="store_true", default=False,
                      help="Dont' ask for confirmation. Just do it.")

    options, args = parser.parse_args(args=args)

    if len(args) != 4:
        parser.error("invalid syntax")

    options.source_ppa = args.pop(0)
    options.dest_ppa = args.pop(0)
    options.series = args.pop(0)
    options.pkg = args.pop(0)
    return options


def default_options(launchpad, options):
    options.source_ppa = ppalib.get_archive(launchpad, options.source_ppa)
    options.dest_ppa = ppalib.get_archive(launchpad, options.dest_ppa)

    options.series = options.source_ppa.distribution.getSeries(name_or_version=options.series)

    if options.to_series:
        options.to_series = options.dest_ppa.distribution.getSeries(name_or_version=options.to_series)
    else:
        options.to_series = options.series


def main():
    apt_pkg.init()

    options = parse_options()

    launchpad = Launchpad.login_with(CONSUMER_KEY,
                                     options.launchpad_instance,
                                     version="devel")

    default_options(launchpad, options)

    kwargs = {'status': 'Published'}
    kwargs['source_name'] = options.pkg
    kwargs['exact_match'] = True
    kwargs['distro_series'] = options.series

    source_pkgs = options.source_ppa.getPublishedSources(**kwargs)
    assert len(source_pkgs) == 1
    source_pkg = source_pkgs[0]

    kwargs['distro_series'] = options.to_series
    current_pkgs = options.dest_ppa.getPublishedSources(**kwargs)
    assert len(current_pkgs) <= 1

    if len(current_pkgs) == 1:
        current_pkg = current_pkgs[0]

        cmpres = apt_pkg.version_compare(current_pkg.source_package_version,
                                         source_pkg.source_package_version)
        if cmpres > 0:
            print("%s is newer in %s (%s) than in %s (%s). Aborting." %
                  (options.pkg, options.dest_ppa.name,
                   current_pkg.source_package_version,
                   options.source_ppa.name,
                   source_pkg.source_package_version))
            return False
        elif cmpres == 0:
            print("%s is the same version in %s/%s and %s/%s. Nothing to do." %
                  (options.pkg,
                   options.source_ppa.name,
                   options.series.name,
                   options.dest_ppa.name,
                   options.to_series.name))
            return True

    print("Will copy %s/%s (%s) from %s to %s/%s." %
          (options.pkg, options.series.name,
           source_pkg.source_package_version,
           options.source_ppa.name, options.dest_ppa.name,
           options.to_series.name))

    if not options.force:
        resp = raw_input('Does this look ok (answer "yes" to confirm")? ')
        if resp == 'yes':
            print("Doing so...")
            options.dest_ppa.syncSource(from_archive=source_pkg.archive,
                                        include_binaries=options.binaries,
                                        source_name=source_pkg.source_package_name,
                                        version=source_pkg.source_package_version,
                                        to_series=options.to_series.name)
            print("Done.")
            return True
        else:
            print("Aborting at your request.")
            return True


if __name__ == '__main__':
    sys.exit(not main())
