#!/usr/bin/env python

# Copyright: (C) 2011 OpenStack LLC
# Copyright: (C) 2012 Nebula Inc.
# Author: Soren Hansen <soren@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class Archive(object):
    def getPublishedSources(self, *args, **kwargs):
        raise NotImplementedError()

    def format_name(self):
        raise NotImplementedError()

    def syncSource(self, *args, **kwargs):
        raise NotImplementedError()

    def getSeries(self, **kwargs):
        return self.archive.distribution.getSeries(**kwargs)


class Distro(Archive):
    def __init__(self, archive, pocket, distro_name):
        self.archive = archive
        self.pocket = pocket
        self.distribution = archive.distribution
        self.name = '%s/%s' % (distro_name, pocket,)


    def getPublishedSources(self, *args, **kwargs):
        kwargs['pocket'] = self.pocket
        return self.archive.getPublishedSources(*args, **kwargs)


    def syncSource(self, *args, **kwargs):
        kwargs['to_pocket'] = self.pocket.capitalize()
        return self.archive.syncSource(*args, **kwargs)


class PPA(Archive):
    def __init__(self, archive):
        self.archive = archive
        self.distribution = archive.distribution
        self.name = format_ppa_name(archive)

    def getPublishedSources(self, *args, **kwargs):
        return self.archive.getPublishedSources(*args, **kwargs)

    def syncSource(self, *args, **kwargs):
        kwargs['to_pocket'] = 'Release'
        return self.archive.syncSource(*args, **kwargs)

    def get_contents(self, series=None):
        """Return contents of PPA

        The return value is a two-dimensional dict.
        First key is the series_name (e.g. "lucid").
        Next key is source_package_name (e.g. "nova").
        Value is the source package object

        Note: Considers both Published and Pending packages
        Note: Intentionally not defined on the Archive superclass,
              in an effort to prevent anyone from accidentally
              calling getPublishedSources without a pkgname on
              the main Ubuntu archive."""

        statuses = ['Published','Pending']
        retval = {}
        distro_series_names = {}
        for status in statuses:
            kwargs = {}
            if series is not None:
                kwargs['distro_series'] = series
            kwargs['status'] = status

            for sp in self.getPublishedSources(**kwargs):
                if sp.distro_series_link not in distro_series_names:
                    distro_series_names[sp.distro_series_link] = sp.distro_series.name
                series_name = distro_series_names[sp.distro_series_link]
                if series_name not in retval:
                    retval[series_name] = {}
                retval[series_name][sp.source_package_name] = sp

        return retval

owner_name_cache = {}
def format_ppa_name(archive):
    global owner_name_cache
    if archive.owner_link not in owner_name_cache:
        owner_name_cache[archive.owner_link] = archive.owner.name

    owner_name = owner_name_cache[archive.owner_link]
    return '%s/%s' % (owner_name, archive.name)


def get_archive(lp, name):
    """Turn a PPA name such as "soren/ppa" into the matching Archive object

    If name is of the form "ubuntu/something", you'll instead get the
    "Something" pocket of the main Ubuntu archive."""

    owner_or_distro, ppaname_or_pocket = name.split('/')
    if owner_or_distro.lower() == 'ubuntu':
        return Distro(lp.projects['ubuntu'].main_archive,
                      ppaname_or_pocket.capitalize(),
                      'Ubuntu')
    elif owner_or_distro.lower() == 'debian':
        return Distro(lp.projects['debian'].main_archive,
                      ppaname_or_pocket.capitalize(),
                      'Debian')
    else:
        ppa = lp.people[owner_or_distro].getPPAByName(name=ppaname_or_pocket)
        return PPA(ppa)
