#!/usr/bin/python

# Copyright (C) 2011  OpenStack LLC
# Copyright (C) 2012  Nebula Inc.
# Author: Soren Hansen <soren@ubuntu.com>
#
# In part based on sync-helper.py,  which is Copyright (C) 2009, 2010,
# 2011  Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Replace the contents of one PPA with that of one or more others."""

from optparse import OptionParser

import apt_pkg
import ppalib
from launchpadlib.launchpad import Launchpad

CONSUMER_KEY = "sync-ppas.py"

def parse_options(args=None):
    parser = OptionParser(usage="Usage: %prog [options] <source PPA> [<another source PPA> [<yet another source PPA]] <destination PPA>")

    parser.add_option("-l", "--launchpad", dest="launchpad_instance",
                      default="production")
    parser.add_option("-s", "--series", dest="series",
                      help="act on packages in SERIES")
    parser.add_option("-i", "--ignore", action="append", metavar="PKG",
                      dest="ignores", help="ignore package PKG")

    options, args = parser.parse_args(args=args)
    if len(args) < 2:
        parser.error("invalid syntax")

    options.dest_ppa = args.pop()
    options.source_ppas = args
    return options


def default_options(launchpad, options):
    options.source_ppas = [ppalib.get_archive(launchpad, name) for name in options.source_ppas]
    options.dest_ppa = ppalib.get_archive(launchpad, options.dest_ppa)

def merge_contents(aggregate, ppa_contents):
    for series in ppa_contents:
        if series not in aggregate:
            aggregate[series] = {}
        for pkg in ppa_contents[series]:
            if pkg in aggregate[series]:
                current_version = aggregate[series][pkg].source_package_version
                candidate_version = ppa_contents[series][pkg].source_package_version
                cmpres = apt_pkg.version_compare(current_version, candidate_version)
                if cmpres < 0:
                    print ("%s had newer %s than %s. Using that instead." %
                           (ppalib.format_ppa_name(ppa_contents[series][pkg].archive),
                           pkg, ppalib.format_ppa_name(aggregate[series][pkg].archive)))
                    print ("%s: %s." %
                           (ppalib.format_ppa_name(aggregate[series][pkg].archive),
                            aggregate[series][pkg].source_package_version))
                    print ("%s: %s." %
                           (ppalib.format_ppa_name(ppa_contents[series][pkg].archive),
                            ppa_contents[series][pkg].source_package_version))
                    aggregate[series][pkg] = ppa_contents[series][pkg] #.source_package_version
            else:
                aggregate[series][pkg] = ppa_contents[series][pkg] #.source_package_version

def detect_removals(source_ppa_contents, dest_ppa_contents):
    removals = {}
    for series in dest_ppa_contents:
        for pkg in dest_ppa_contents[series]:
            if (series not in source_ppa_contents or
                pkg not in source_ppa_contents[series]):
                if series not in removals:
                    removals[series] = []
                removals[series] += [dest_ppa_contents[series][pkg]]
    return removals

def count(d):
    return sum([len(d[s]) for s in d], 0)

def main():
    apt_pkg.init()

    options = parse_options()

    launchpad = Launchpad.login_with(CONSUMER_KEY,
                                     options.launchpad_instance,
                                     version="devel")

    default_options(launchpad, options)

    aggregate_source_contents = {}
    for ppa in options.source_ppas:
        source_ppa_contents = ppa.get_contents()
        merge_contents(aggregate_source_contents, source_ppa_contents)

    dest_ppa_contents = options.dest_ppa.get_contents()

    if options.ignores:
        for contents in (aggregate_source_contents, dest_ppa_contents):
            for series in contents:
                contents[series] = dict((name,pkg) for (name,pkg) in contents[series].iteritems() if name not in options.ignores)

    removals = detect_removals(aggregate_source_contents, dest_ppa_contents)

    print "Removals:"
    if removals:
        print ("The following packages are not in any of the source PPA's\n"
               "anymore and will be removed in %s:" % (options.dest_ppa.name))
        for series in removals:
            print "  %s:" % (series,)
            for pkg in removals[series]:
                print ('    %s-%s (%s)' %
                       (pkg.source_package_name,
                        pkg.source_package_version,
                        series))
    else:
        print "  No removal candidates found"

    print "\n"

    syncs = {}
    for series in aggregate_source_contents:
        for pkg in aggregate_source_contents[series]:
            if (series not in dest_ppa_contents
                or pkg not in dest_ppa_contents[series]):
                if series not in syncs:
                    syncs[series] = []
                syncs[series] += [(aggregate_source_contents[series][pkg], 'absent')]
            elif (apt_pkg.version_compare(aggregate_source_contents[series][pkg].source_package_version,
                                          dest_ppa_contents[series][pkg].source_package_version) > 0):
                if series not in syncs:
                    syncs[series] = []
                syncs[series] += [(aggregate_source_contents[series][pkg], dest_ppa_contents[series][pkg].source_package_version)]

    print "Syncs:"
    if syncs:
        print ("The following packages will be copied as they are either\n"
               "newer in one of the source PPA's than in %s\nor not present in %s at all." %
               (options.dest_ppa.name,
                options.dest_ppa.name))
        print "Format is: <source package name> - <new version> (<current version>)"
        for series in syncs:
            print "  %s:" % (series,)
            fmt = "    %-20s - %s (%s)"
            for (pkg, previous_version) in syncs[series]:
                print fmt % (pkg.source_package_name, pkg.source_package_version, previous_version)
    else:
        print "  No candidates for syncing found"

    if not syncs and not removals:
        return True
    else:
        print ""

    failures = []
    resp = raw_input('%d syncs, %d removals. Does this look ok '
                     '(answer "yes" to confirm")? ' %
                     (count(syncs), count(removals)))
    if resp == 'yes':
        for series in removals:
            for pkg in removals[series]:
                print ('  Removing %s-%s from %s (%s) ... ' %
                       (pkg.source_package_name,
                        pkg.source_package_version,
                        options.dest_ppa.name, series))
                pkg.requestDeletion(removal_comment="Removed from staging PPA(s)")

        for series in syncs:
            for (pkg, previous_version) in syncs[series]:
                print ('  Syncing %s-%s (%s) from %s to %s... ' %
                       (pkg.source_package_name,
                        pkg.source_package_version,
                        series, ppalib.format_ppa_name(pkg.archive),
                        options.dest_ppa.name))

                try:
                    options.dest_ppa.syncSource(from_archive=pkg.archive,
                                                include_binaries=True,
                                                source_name=pkg.source_package_name,
                                                version=pkg.source_package_version,
                                                to_series=series) # Yes, to_series is necessary if there are binaries that were copied from series to another in the source_ppa.
                except Exception, e:
                    failures += ['Failure trying to sync %s: %s' % (pkg.source_package_name, str(e))]
                    print "Failed!"
                    continue
                print "Done!"
    for failure in failures:
        print failure

if __name__ == '__main__':
    main()
