#!/usr/bin/python

# Copyright (C) 2012  Nebula Inc.
# Author: Soren Hansen <soren@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Compare contents of Ubuntu archives with that of a PPA"""

from optparse import OptionParser

import apt
import apt_pkg
from launchpadlib.launchpad import Launchpad
import os.path
import ppalib

apt_pkg.init()
CONSUMER_KEY = "staging-ppa-status.py"

def parse_options(args=None):
    parser = OptionParser(usage="Usage: %prog [options] <PPA> <series> <pkglist>")

    parser.add_option("-l", "--launchpad", dest="launchpad_instance",
                      default="production")
    parser.add_option("-d", "--debug", default=False, action="store_true")
    parser.add_option("-t", "--table", default=False, action="store_true",
                      help="Show output as a table")
    parser.add_option("-f", "--full", default=False, action="store_true",
                      help="Include all packages in the table (no-op if --table isn't set)")
    parser.add_option("--html", default=False, action="store_true",
                      help="Output table as HTML (no-op if --table isn't set)")

    options, args = parser.parse_args(args=args)
    if len(args) != 3:
        parser.error("invalid syntax")

    options.ppa = args.pop(0)
    options.series = args.pop(0)
    options.pkglist = args.pop(0)
    return options, args


def default_options(launchpad, options):
    options.source_ppa = ppalib.get_archive(launchpad, options.ppa)

    ubuntu = launchpad.projects['ubuntu']
    options.ubuntu_archive = ubuntu.getArchive(name='primary')
    options.series_obj = ubuntu.getSeries(name_or_version=options.series)

def get_ppa_contents(launchpad, ppa, series):
    statuses = ['Published','Pending']
    retval = {}

    # Doing Published pkgs first, then Pending. This way, we shouldn't
    # have to worry about version checking as the pending package must
    # be newer than the published one.
    for status in statuses:
        for sp in ppa.getPublishedSources(status=status, distro_series=series):
            retval[sp.source_package_name] = sp

    return retval

def get_pkg_list(filename):
    pkg_list = []
    for l in open(filename, 'r'):
        pkg_list.append(l.strip())
    return pkg_list

def get_version_info(options, rootdir, ppa_contents):
    version_info = []

    cache = apt.Cache(rootdir=rootdir)
    try:
        cache.update()
        cache.open()
    except apt.cache.FetchFailedException:
        pass

    srcrecords = apt_pkg.SourceRecords()

    for pkgname in get_pkg_list(options.pkglist):
        if options.debug:
            print 'Examining %s' % (pkgname,)

        srcrecords.restart()

        release_version = None
        updates_version = None
        security_version = None
        proposed_version = None
        ppa_version = None

        while srcrecords.Lookup(pkgname):
            archive = srcrecords.index.describe.split(' ')[1].split('/')[0]

            if '-' not in archive:
                release_version = srcrecords.version
            elif '-updates' in archive:
                updates_version = srcrecords.version
            elif '-security' in archive:
                security_version = srcrecords.version
            elif '-proposed' in archive:
                proposed_version = srcrecords.version

        if pkgname in ppa_contents:
            ppa_version = ppa_contents[pkgname].source_package_version

        t = (pkgname,
             ppa_version,
             release_version,
             security_version,
             proposed_version,
             updates_version)
        version_info.append(t)

    return version_info

def cmp_version(a, b):
    return apt_pkg.VersionCompare(a or '', b or '')

def min_version(*l):
    v = None
    for candidate in l:
        if v is None:
            v = candidate
        elif cmp_version(v, candidate) > 0:
            v = candidate
    return v

def max_version(*l):
    v = None
    for candidate in l:
        if cmp_version(v, candidate) < 0:
            v = candidate
    return v

def update_location(l):
    (pkg_name, ppa_version, release_version, security_version,
                            proposed_version, updates_version) = l

    current_version = max(ppa_version, release_version)

    if cmp_version(updates_version, current_version) > 0:
        return 'updates'
    if cmp_version(security_version, current_version) > 0:
        return 'security'
    if cmp_version(proposed_version, current_version) > 0:
        return 'proposed'

    return False

def main():
    options, args = parse_options()

    launchpad = Launchpad.login_with(CONSUMER_KEY,
                                     options.launchpad_instance,
                                     version="devel")

    default_options(launchpad, options)

    apt_root = os.path.abspath(options.series)
    if not os.path.exists(apt_root):
        etc_apt = os.path.join(apt_root, 'etc', 'apt')
        os.makedirs(etc_apt)
        archive = 'http://archive.ubuntu.com/ubuntu'
        with open(os.path.join(etc_apt, 'sources.list'), 'w') as f:
            for pocket in ['', '-updates', '-security']:
                f.write('deb %s %s%s main universe restricted multiverse\n' %
                        (archive, options.series, pocket))
                f.write('deb-src %s %s%s main universe restricted multiverse\n' %
                        (archive, options.series, pocket))

            f.write('deb http://ppa.launchpad.net/%s/ubuntu %s main\n' %
                    (options.ppa, options.series))
            f.write('deb-src http://ppa.launchpad.net/%s/ubuntu %s main\n' %
                    (options.ppa, options.series))

    ppa_contents = options.source_ppa.get_contents(series=options.series_obj)
    ppa_contents = ppa_contents[options.series]

    version_info = get_version_info(options, apt_root, ppa_contents)

    if options.table:
        from prettytable import PrettyTable
        table = PrettyTable(["Source", "PPA version", "Release", "Security", "Proposed", "Updates", "Needs update"])
        table.set_field_align("Source", "l")
        for l in version_info:
            row = list(l) + [update_location(l) and 'Yes' or 'No']
            if update_location(l) or options.full:
                table.add_row(row)
        if options.html:
            print table.get_HTML_string()
        else:
            print table
    else:
        for l in version_info:
            if update_location(l):
                print('./copy-ppa-pkg.py ubuntu/%s %s %s %s' % (update_location(l), options.ppa, options.series, l[0]))

if __name__ == '__main__':
    main()
else:
    import unittest

    class StuffTests(unittest.TestCase):
        def test_min_version(self):
            self.assertIsNone(min_version())
            self.assertEquals(min_version('0'), '0')
            self.assertEquals(min_version('1'), '1')
            self.assertEquals(min_version('1', '2'), '1')
            self.assertEquals(min_version('2', '1'), '1')
            self.assertEquals(min_version('3', '2', '1'), '1')
            self.assertEquals(min_version('2', '3', '1'), '1')
            self.assertEquals(min_version('2', '1', '3'), '1')
            self.assertEquals(min_version('2', '1', '3'), '1')
    
        def test_max_version(self):
            self.assertIsNone(max_version())
            self.assertEquals(max_version('0'), '0')
            self.assertEquals(max_version('1'), '1')
            self.assertEquals(max_version('1', '2'), '2')
            self.assertEquals(max_version('2', '1'), '2')
            self.assertEquals(max_version('3', '2', '1'), '3')
            self.assertEquals(max_version('2', '3', '1'), '3')
            self.assertEquals(max_version('2', '1', '3'), '3')
            self.assertEquals(max_version('2', '1', '3'), '3')
    
        def test_update_location(self):
            # We need to update the package if:
            #   It's not in the PPA at all, and there's something in either of updates or security
            self.assertEquals(update_location(('foo', None, '1.1', '1.2', None, None)), 'security',
                              'Package not in PPA, but with security update did not trigger update_location')
            self.assertEquals(update_location(('foo', None, '1.1', None, None, '1.2')), 'updates',
                            'Package not in PPA, but with important update did not trigger update_location')
            self.assertEquals(update_location(('foo', '1.2', '1.1', None, None, '1.3')), 'updates',
                            'Package in PPA, but with important update did not trigger update_location')
            self.assertEquals(update_location(('foo', '1.2', '1.1', '1.3', None, None)), 'security',
                            'Package in PPA, but with security update did not trigger update_location')
    
            self.assertFalse(update_location(('foo', None, '1.1', None, None, None)),
                            'Package not in PPA, no updates, yet update_location return True')
    
            self.assertFalse(update_location(('foo', '0.8.3-1ubuntu19.4.0~nebula1', '0.8.3-1ubuntu14',
                                              '0.8.3-1ubuntu19.1', None, '0.8.3-1ubuntu19.4')),
                            'Package in PPA, but with security update did not trigger update_location')
